/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#ifndef __CALIB_AGC__
#define __CALIB_AGC__

class Calibration_AGC
{
public:
	struct Parameters
	{
		float th_lower, th_upper;
		float adj_lower, adj_upper;
		unsigned int th_count;
	};

	typedef std::vector<float> LuminanceVec;
	typedef std::vector<std::pair<float, size_t>> DiffVec;
	
public:
	Calibration_AGC(Scenario::AGCMethod method);
	~Calibration_AGC(void);

	// Read reference video
	void openRefVideo(std::string file);
	bool readRefVideo(float &progress);

	// Read copy video
	void openCopyVideo(std::string file);
	bool readCopyVideo(float &progress);

	// Serialization
	void writeRefLuminancesToFile(std::string file);
	void writeCopyLuminancesToFile(std::string file);
	bool readRefLuminancesFromFile(std::string file);
	bool readCopyLuminancesFromFile(std::string file);

	// Registration
	void startRegistration();
	bool doRegistrationStep(float &progress, int interval);
	void finishRegistration(std::string result_file);

protected:
	float calculateLuminance(cv::Mat &frame);
	
	float calculateDiff(const LuminanceVec &data1,
						const LuminanceVec &data2);

	void applyParameters(
		const LuminanceVec &orig_luminance,
		LuminanceVec &mod_luminance,
		const Parameters &params);
	
	void threadApplyParameters(
		size_t start, size_t stop, int index);

protected:
	VideoReader ref, copy;

	LuminanceVec ref_luminance;
	LuminanceVec copy_luminance;

protected:
	float lowestDiff;

	Scenario::AGCMethod method;

	float THRESH_RANGE, THRESH_STEP;
	float ADJUST_RANGE, ADJUST_STEP;
	unsigned int COUNT_RANGE;

	std::vector<Parameters> paramIndexList;
	
private:
	unsigned int numFrames;
	unsigned int currFrame;

	size_t params_list_start;
	size_t bestParamsIndex;
	size_t currStep;
	
	DiffVec difference;
	
	boost::mutex curr_frame_mutex;
	
	cv::Mat frame, frame_ycrcb;
	std::vector<cv::Mat> xyzChannels;

	const int numCores;
};

#endif //__CALIB_AGC__