/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#ifndef __CALIB_SPATIAL__
#define __CALIB_SPATIAL__

class Calibration_Spatial
{
public:
	// Limits for the number of features maximally used
	static const unsigned int maxNumFeatures_Persp = 5000;
	static const unsigned int maxNumFeatures_Lens = 15000;

public:
	Calibration_Spatial();
	~Calibration_Spatial(void);

	// Initial step
	void startRegistration(std::string ref_file, std::string copy_file,
						   bool checkerboardPattern, bool assumeLensDistortion,
						   std::string result_file);
	// Processing steps
	void doRegistrationStep();
	
	// Finishing step
	void finishRegistration();
	bool finishedProcessing ; 

	float getProgress() ;
	bool doneProcessing() ;
	
	int maxInQueueSize ;
	int maxOutQueueSize ;

	inline bool hasPendingLogMessages()
	{	return ThreadCout::tcout.getPendingMsgCount() > 0;	}

	inline void showLogMessages(std::ostream &os = std::cout)
	{	ThreadCout::tcout.render(os, true);	}
	
protected:    
	VideoReader ref, copy;
	cv::FileStorage fs_result;

	bool checkerboardPattern;
	bool assumeLensDistortion;
	
	struct FrameData
	{
		int idx ;
		cv::Mat frame ;
		ImRegParams sr ;
	};
	
	struct FramePair
	{
		unsigned int idx ; 
		cv::Mat refFrame ;
		cv::Mat cpyFrame ;
	};
	
	unsigned int currentInputFrame ; 

private:
	boost::thread_group processFrames ;
	boost::thread_group writeResults ;
		
	SynchronizedQueue<FramePair> queueIn ;
	SynchronizedQueue<FrameData> queueOut ;
	
	void loadVideoQueues() ;
	void writeRegistrationResults( cv::VideoWriter& writer ) ;

	unsigned int numFrames;
	std::atomic<unsigned int> currFrame;
	
	unsigned int pairMode ;
	
	int numThreads ; 
};

#endif //__CALIB_SPATIAL__