/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#ifndef __CALIB_TEMPORAL__
#define __CALIB_TEMPORAL__

class Calibration_Temporal
{
public:
	Calibration_Temporal(void);
	~Calibration_Temporal(void);
	
	typedef std::pair<int, cv::Mat> FrameWithIdx;

	/* Functions */
	
	void startRegistration(std::string ref_file);
	void finishRegistration() ;

	bool doneProcessing() ;
	float getProgress() ;
	
	inline bool hasPendingLogMessages()
	{	return ThreadCout::tcout.getPendingMsgCount() > 0;	}

	inline void showLogMessages(std::ostream &os = std::cout)
	{	ThreadCout::tcout.render(os, true);	}
	
	/* Variables */
	
	int maxQueueOutSize ;
	int maxQueueInSize  ;

protected:
	void findSpatialAlignment(cv::Mat syncPattern, std::string spatialDeformFile) ;
	void loadMarkerData(const std::string filename, std::string &syncPatternFile, cv::Size &markerVideoSize);
	void loadLUT_pixelVal2Exp(std::string lut_file);

	void loadVideoQueues() ;
	void encodeFrame( cv::VideoWriter& writer ) ;
	void processFrame() ;

	std::pair<int, float> readFrameNumber(
		cv::Mat &background, std::vector<cv::Rect> markers, float confThreshold);

	std::pair<unsigned int, float> readXOMarkers(
		cv::Mat &background, std::vector<cv::Rect> positions);

	std::vector<std::vector<double>> scanMarkers (
		cv::Mat background, unsigned int numHorSamplingSteps);

	std::vector<std::vector<double>> calcInfluenceFromMarkers (
		std::vector<std::vector<double>> markerData,
		bool frame1Valid, bool frame2Valid, bool frame3Valid);

	void subtractBlackValue(std::vector<std::vector<double>> &data);
	void applyLUTtoMarkers(std::vector<std::vector<double>> &data);

	void writeBlendDataAndAvg(
		const std::vector<std::vector<double>> &data,
		std::vector<int> frameNumbers, unsigned int realFrameIdx,
		std::ofstream &dataStream);

private:
	/* Reader and Writer */
	VideoReader input_vid ;
	
	/* Instances */
	boost::thread_group processFrames ;
	
	// Spatial distortion
	ImRegParams deform;
	ImRegParams redeform;

	cv::Mat imgX, imgO;
	
	/* Variables */
	
	SynchronizedQueue<FrameWithIdx> frameQueueIn ;
	SynchronizedQueue<FrameWithIdx> frameQueueOut ;

	std::string outputFolder ;

	std::ofstream avgBlendData;
	
	cv::Mat deformedMask ;
	int firstFrameAfterSyncPattern;

	cv::Rect centerArea;
	std::vector<cv::Rect> markersTop[3];
	std::vector<cv::Rect> markersBottom[3];
	cv::Rect markersLeft[5];
	cv::Rect markersRight[5];
	
	int lastValidFrameNumber ;
	int maxValidFrameJump ;

	cv::Mat lut_gamma;
	std::vector<std::vector<double>> lut_pixelVal2Exp;

#ifdef EXTRACT_LUMINANCES
	unsigned int acumulatedBlendDataCount;
#endif

	std::vector<std::vector<double>> acumulatedBlendData;

	int currFrame ; 
	
	bool finishedProcessing ;
	int totalFrames ;
	int numThreads ;
	
	bool hasCUDASupport;
};

#endif //__CALIB_TEMPORAL__

