/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#ifndef __CALIB_TEMPORAL_MARKER__
#define __CALIB_TEMPORAL_MARKER__

class Calibration_Embed
{   
public:
	Calibration_Embed(void);
	~Calibration_Embed(void);
	
	typedef std::pair<int, cv::Mat> FrameWithIdx;

	/**** Functions ****/
	
	void startRegistration( std::string src_file, std::string dst_file,
							std::string pat_file, bool createLuminanceTransferVideo = false) ;
	
	void finishRegistration() ; 

	float getProgress() ;    
	bool doneProcessing() ;
	
	inline bool hasPendingLogMessages()
	{	return ThreadCout::tcout.getPendingMsgCount() > 0;	}

	inline void showLogMessages(std::ostream &os = std::cout)
	{	ThreadCout::tcout.render(os, true);	}
		
	/**** Variables ****/
	int maxQueueInSize ;
	int maxQueueOutSize ;
	
protected:
	void processFrame() ;
	
private:
	/**** Variables ****/
	
	bool createLuminanceTransferVideo;

	int currFrame ;
	bool finishedProcessing ;
	int totalFrames ;
	int numThreads ; 
	
	cv::Mat imgX, imgO;

	cv::Size outVideoSize;

	cv::Rect centerArea;
	std::vector<cv::Rect> markersTop[3];
	std::vector<cv::Rect> markersBottom[3];
	cv::Rect markersLeft[5];
	cv::Rect markersRight[5];
	
	/**** Thread Objects ****/
	
	boost::thread_group processFrames ;
	boost::thread_group encodeVideo ;
	SynchronizedQueue<FrameWithIdx> frameQueueIn ;
	SynchronizedQueue<FrameWithIdx> frameQueueOut ;
	
	/**** Video Reader ****/
	
	VideoReader input_vid ;
	
	/**** Functions ****/
	
	cv::Size calculateMarkerPositions(
		cv::Size srcFrameSize,
		unsigned int symbolCount, cv::Size symbolSize);

	void saveMarkerData(const std::string filename, const std::string syncPatternFile);

	void drawXOMarkers(cv::Mat &background, std::vector<cv::Rect> positions, unsigned int decimal);

	void writeSyncFrames ( int width, int height, cv::VideoWriter& writer,
						   std::string destSyncPattern, unsigned int frameCount = 5 ) ;

	void writeVideo(cv::VideoWriter &_writer) ;
	void loadVideoQueues() ; 
};

#endif //__CALIB_TEMPORAL_MARKER__
