/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../stdafx.h"

#include "../utility/Version.h"

#include "../common/Scenario.h"

#include "SimAGC.h"

using namespace std;
using namespace cv;

Simulation_AGC::Simulation_AGC()
	: cnt_lower(0), cnt_upper(0),
	  lum_last(0), gain_offset(0.0f),
	  ycrcbChannels(3)
{
}

Simulation_AGC::~Simulation_AGC(void)
{
}

void Simulation_AGC::setup(
	Scenario::AGCMethod _method,
	unsigned int _frameCountLower, unsigned int _frameCountUpper,
	float _threshLower, float _threshUpper,
	float _adjustLower, float _adjustUpper)
{
	method = _method;
	frameCountLower = _frameCountLower;
	frameCountUpper = _frameCountUpper;
	threshLower = _threshLower; threshUpper = _threshUpper;
	adjustLower = _adjustLower; adjustUpper = _adjustUpper;
}

void Simulation_AGC::processFrame(cv::Mat &frame)
{
	// Convert and split channels
	cvtColor( frame, frame_ycrcb, CV_BGR2YCrCb );
	split( frame_ycrcb, ycrcbChannels );

	// Count frames in which luminance is outside threshold.
	// Reset this count to 0 once we are within the threshold again.
	// If outside threshold for several frames, change gain.

	// Lower bound
	if( lum_last > threshLower ) cnt_lower = 0;
	else
	{
		if( cnt_lower >= frameCountLower )
		{
			switch(method)
			{
			case Scenario::AGC_MEAN_THRESHOLD_OFFSET_FIX_ADJUST:
				gain_offset += adjustLower; break;
	
			case Scenario::AGC_MEAN_THRESHOLD_OFFSET_PERC_ADJUST:
				gain_offset += (threshLower - lum_last) * adjustLower; break;
			}
		}
			
		else cnt_lower++;
	}

	// Upper bound
	if( lum_last < threshUpper ) cnt_upper = 0;
	else
	{
		if( cnt_upper >= frameCountUpper )
		{
			switch(method)
			{
			case Scenario::AGC_MEAN_THRESHOLD_OFFSET_FIX_ADJUST:
				gain_offset -= adjustUpper; break;
	
			case Scenario::AGC_MEAN_THRESHOLD_OFFSET_PERC_ADJUST:
				gain_offset += (threshUpper - lum_last) * adjustUpper; break;
			}
		}
		else cnt_upper++;
	}

	// Adjust luminance by gain
	ycrcbChannels[0] += gain_offset;

	// Get average frame luminance
	lum_last = (float)mean(ycrcbChannels[0])[0];

	// Merge channels and convert back
	merge( ycrcbChannels, frame_ycrcb);
	cvtColor( frame_ycrcb, frame, CV_YCrCb2BGR);
}