/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../stdafx.h"

#include "SimBayerCFA.h"

using namespace std;
using namespace cv;

void Simulation_BayerCFA::applyAAFilter(cv::Mat &frame)
{
	// Apply a 2x2 box filter as anti-aliasing filter
	boxFilter(frame, frame, -1, Size(2,2));
}

bool Simulation_BayerCFA::applySimpleBayerCFA(cv::Mat &frame)
{
	if(frame.channels() != 3)
		return false;
	
	Size size = frame.size();
	int width = size.width;
	int height = size.height;

	Mat bayer;
	bayer = Mat::zeros( height, width, CV_8UC1 );

	for (int y=0; y<height; y++ )
	{
		if (y%2 == 0)
		{
			// bayer: rgrgrgrg -> v=2,1,2,1,2,1,2...
			for (int x=0; x<width; x++ )
			{
				if (x%2 == 0)	bayer.at<uchar> (y,x) = frame.at<Vec3b> (y,x) [2];
				else			bayer.at<uchar> (y,x) = frame.at<Vec3b> (y,x) [1];
			}
		}
		else
		{
			// bayer: gbgbgbgbgb -> v=1,0,1,0,1,0...
			for (int x=0; x<width; x++ )
			{
				if (x%2 == 0)	bayer.at<uchar> (y,x) = frame.at<Vec3b> (y,x) [1];
				else			bayer.at<uchar> (y,x) = frame.at<Vec3b> (y,x) [0];
			}
		}
	}

	Mat dest(height, width, CV_8UC3);
	cvtColor(bayer, dest, CV_BayerBG2BGR );

	frame = dest;

	return true;
}
