/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../stdafx.h"

#include "../utility/Util.h"

#include "SimDisplayPWM.h"

using namespace std;
using namespace cv;

// Static member initialization
double Simulation_DisplayPWM::pwmHighValue = 1.0;

Simulation_DisplayPWM::Simulation_DisplayPWM()
	: pwmDutyCycle(1.0), pwmDuration(1000.0 / 180.0)
{
}

void Simulation_DisplayPWM::setup(
	double pwmDutyCycle, double pwmFrequency)
{
	this->pwmDutyCycle = pwmDutyCycle;
	this->pwmDuration = 1000.0 / pwmFrequency;
}

double Simulation_DisplayPWM::integrate(
	double startTime, double endTime) const
{
	// Simple corner case
	if(pwmDutyCycle == 1.0)
		return (endTime - startTime) * pwmHighValue;

	// Find period in which time span starts/ends
	unsigned int startPeriodIdx = (unsigned int)(startTime / pwmDuration);
	unsigned int endPeriodIdx = (unsigned int)(endTime / pwmDuration);
	int numPeriods = endPeriodIdx - startPeriodIdx + 1;

	double value = 0.0;

	switch(numPeriods)
	{
	case 1:
		// Integrate over the one period in which the time span starts and ends
		value += integrate_one_period(startPeriodIdx, startTime, endTime);
		break;

	default:
		// Integrate all the periods in between the start and end period
		value += ((numPeriods - 2) * pwmDutyCycle * pwmDuration * pwmHighValue);
		// No break here since in this case we also need to perform the operations from case 2.

	case 2:
		// Integrate over the two periods in which the time span starts, resp. ends
		value += integrate_start_period(startPeriodIdx, startTime);
		value += integrate_end_period(endPeriodIdx, endTime);
		break;
	}

	return value;
}

double Simulation_DisplayPWM::integrate_one_period(
	unsigned int periodIdx, double startTime, double endTime) const
{
	double periodStart = pwmDuration * periodIdx;
	double periodTime_Start = startTime - periodStart;
	double periodTime_End = endTime - periodStart;
	double periodPowerDownTime = pwmDutyCycle * pwmDuration;

	return (periodTime_Start < periodPowerDownTime) ?
		   (min(periodPowerDownTime, periodTime_End) - periodTime_Start) * pwmHighValue : 0.0;
}

double Simulation_DisplayPWM::integrate_start_period(
	unsigned int periodIdx, double time) const
{
	double periodStart = pwmDuration * periodIdx;
	double periodTime = time - periodStart;
	double periodPowerDownTime = pwmDutyCycle * pwmDuration;

	return (periodTime < periodPowerDownTime) ?
		   (periodPowerDownTime - periodTime) * pwmHighValue : 0.0;
}

double Simulation_DisplayPWM::integrate_end_period(
	unsigned int periodIdx, double time) const
{
	double periodStart = pwmDuration * periodIdx;
	double periodTime = time - periodStart;
	double periodPowerDownTime = pwmDutyCycle * pwmDuration;

	return (periodTime < periodPowerDownTime) ?
			periodTime * pwmHighValue : periodPowerDownTime * pwmHighValue;
}
