/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#ifndef __SIM_SPATIAL__
#define __SIM_SPATIAL__

class CamMark ; 

class Simulation_Spatial
{
	struct SpatialRegWithBg
	{
		ImRegParams reg_params;
		std::vector<uchar> pngCompressedBgImg;
	};

public:
	Simulation_Spatial(void);
	~Simulation_Spatial(void);
	
	void setup(std::string transform_file, cv::Size frame_size,
			   cv::Size displayResolution, cv::Size cameraResolution,
			   Scenario::AspectRatioMode ar_mode,
			   unsigned int extendBorderPx,
			   unsigned int fadeBorderPx);

	cv::Size getFinalDisplayResolution()	{	return displayResolution;	}
	cv::Size getFinalCameraResolution()		{	return cameraResolution;	}

	void loadBgVideoToInMemPNGs();

	cv::Mat distort(cv::Mat inputFrame, double time) const;

protected:
	void loadIdentityTransformation(cv::Size initializationSize);
	void readTransformFile(std::string spatialTransformFile);

protected:
	// Utility methods
	static void fillMissing(
		cv::Size &input_size, const cv::Size &ref_size,
		double ar = 0 /* if zero, ref's ar is used */);

	static void createDisplayFrame(
		cv::Mat &src, const cv::Size display_res,
		Scenario::AspectRatioMode ar_mode, cv::Mat &out);

	static void addExtrapolatedBorder(
		const cv::Mat &src, unsigned int borderWidthPx, cv::Mat &out);

	static void darkenBorder(
		cv::Mat &image, unsigned int borderWidthPx);

protected:
	cv::Size displayResolution;
	cv::Size cameraResolution;

	Scenario::AspectRatioMode aspect_ratio_mode;
	
	unsigned int extendBorderPx;
	unsigned int fadeBorderPx;

	std::string transformFile;
	std::string bgFile;

	double samplingFrameRate;
	std::vector<SpatialRegWithBg> spatial_registration;
	std::vector<unsigned int> loop_indices;

private:
	bool hasCUDASupport;
	static double fallbackSamplingFrameRate;
};

#endif // __SIM_SPATIAL__