/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#ifndef __SIM_TEMPORAL__
#define __SIM_TEMPORAL__

class Simulation_Temporal
{
public:
	struct SrcFrameInfluence
	{
		unsigned int number;
		float share, power;
	};

	typedef std::vector<SrcFrameInfluence> RowBlendDescription;
	typedef std::vector<RowBlendDescription> BlendDescription;

public:
	Simulation_Temporal(void);
	~Simulation_Temporal(void);

	void setup(
		double src_start_time, double src_end_time,
		double cam_start_time, double cam_end_time,
		double src_fps, double cam_fps,
		double display_refresh_rate, double display_g2g_time,
		double exposure_time_ms, double readout_time_ms,
		bool vsync, std::string dataSrcPathStr);

	void setDisplayPWM(Simulation_DisplayPWM &displayPWM)
	{	this->displayPWM = displayPWM;	}

	unsigned int getCamFrameCount();

	BlendDescription getBlendDescForFrame(unsigned int nbr,
										  unsigned int row_count) const;

	static std::set<unsigned int> getInvoledFrameNumbers(
		BlendDescription blendDesc);

	static cv::Mat blendFrames(const BlendDescription &blendDesc,
							   const std::map<unsigned int, cv::Mat> &srcFrames);

protected:
	void createSourceFrameSequence(
		double src_fps, double start_time, double end_time,
		int vsync, double display_refresh_rate, double display_g2g_time);

	static void simulate_vsync(
		double& time, double display_refresh_rate);

	static std::pair<float, float> getInfluenceFactors(
		double exposure_start_time, double exposure_end_time,
		const Simulation_DisplayPWM &displayPWM,
		const Frame &src_frame);

protected:
	boost::filesystem::path dataSrcPath;

	double cam_fps;
	double cam_start_time, cam_end_time;
	double exposure_time, readout_time;

	Simulation_DisplayPWM displayPWM;

	FrameSequence<Frame> src_frame_seq;

private:
	bool hasCUDASupport;
};

#endif // __SIM_TEMPORAL__
