/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#ifndef __BUFFEREDVIDEOREADER__
#define __BUFFEREDVIDEOREADER__

class BufferedVideoReader : public VideoReader
{
	struct BufferedFrame
	{
		// Constructor for initialization 
		BufferedFrame() : frame_nr(0xFFFFFFFF)	{	}

		cv::Mat frame;
		unsigned int frame_nr;
	};

protected:
	typedef std::vector<BufferedFrame> FrameVector;

public:
	BufferedVideoReader(FrameVector::size_type buffer_size);
	BufferedVideoReader(int device, FrameVector::size_type buffer_size);
	BufferedVideoReader(const std::string &filename, FrameVector::size_type buffer_size);
	virtual ~BufferedVideoReader(void);
	
	virtual unsigned int seekToFrame(unsigned int frame_nr);
	
	virtual cv::Mat getFrame(unsigned int frame_nr);
	virtual cv::Mat getFrame(unsigned int frame_nr, FrameVector &buffer);

	inline unsigned int getNextFrameNumber()
	{	return nextFrameNr;	}

protected:
	virtual bool readNextFrame();

protected:
	unsigned int nextFrameNr;
	FrameVector default_buffer;
};

#endif // __BUFFEREDVIDEOREADER__