/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

//
// This code is based on an implementation by Rajiv Chakravorty,
// published on his website as part of an H.324M Software Simulator project.
// URL: http://www.rajivchakravorty.com/source-code/uncertainty/multimedia-sim/html/crc8_8c-source.html
//

#include "../stdafx.h"

#include "CRC8.h"

#define GP  0x107   /* x^8 + x^2 + x + 1 */
#define DI  0x07

CRC8::CRC8(void)
{
	int i,j;
	unsigned char crc;

	for (i = 0; i < 256; i++) {
		crc = i;
		for (j = 0; j < 8; j++)
			crc = (crc << 1) ^ ((crc & 0x80) ? DI : 0);
		crc8_table2[i] = crc & 0xFF;
	}
}

void CRC8::crc8(unsigned char *crc, unsigned char m)
{
	*crc = crc8_table2[(*crc) ^ m];
	*crc &= 0xFF;
}

unsigned int CRC8::append_24bit_8bitcrc(unsigned int data24bit)
{
	unsigned int result = data24bit & 0x00FFFFFF;
	if(result != data24bit) throw std::runtime_error("ERROR: CRC8::append_24bit_8bitcrc(unsigned int data24bit):\n"
		"Some of the high-order 8bit of the 32bit data parameters are set, but the data must only be 24 bit.");

	unsigned char *resultPtr = (unsigned char*)&result;

	unsigned char crc = 0;
	crc8(&crc, *(resultPtr + 0));
	crc8(&crc, *(resultPtr + 1));
	crc8(&crc, *(resultPtr + 2));
	*(resultPtr + 3) = crc + 1;

	return result;
}
	
bool CRC8::check_24bit_8bitcrc(unsigned int data24bit_8bitcrc)
{
	unsigned char *dataPtr = (unsigned char*)&data24bit_8bitcrc;

	unsigned char crc = 0;
	crc8(&crc, *(dataPtr + 0));
	crc8(&crc, *(dataPtr + 1));
	crc8(&crc, *(dataPtr + 2));
	crc8(&crc, *(dataPtr + 3) - 1);

	return crc == 0;
}
