/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../stdafx.h"

#include "CvUtil.h"

using namespace std;
using namespace cv;

void CvUtil::blendWithMask(
	const Mat &base, const Mat &src, const Mat &mask, Mat &out)
{
	CV_DbgAssert(mask.channels() == 1);

	char ch = base.channels();
	float alpha = 0;
	
	for(int y = 0; y < base.rows; y++)
	{
		const uchar* pBase = base.ptr<uchar>(y);
		const uchar* pSrc  = src.ptr<uchar>(y);
		const uchar* pMask = mask.ptr<uchar>(y);
			  uchar* pOut  = out.ptr<uchar>(y);
		
		for(int x = 0; x < base.cols*ch; x++)
		{
			int ix = x / ch;
			
				 if(pMask[ix] == 255) pOut[x] = pBase[x];
			else if(pMask[ix] == 0)   pOut[x] = pSrc[x];
			else
			{
				alpha = pMask[ix] / 255.0f;
				pOut[x] = uchar(pSrc[x] * (1-alpha) + pBase[x] * alpha);
			}
		}
	}
}

void CvUtil::multiplyWithMask(
	const Mat &base, const Mat &mask, Mat &out)
{
	CV_DbgAssert(mask.channels() == 1);

	char ch = base.channels();
	float alpha = 0;
	
	for(int y = 0; y < base.rows; y++)
	{
		const uchar* pBase = base.ptr<uchar>(y);
		const uchar* pMask = mask.ptr<uchar>(y);
			  uchar* pOut  = out.ptr<uchar>(y);
		
		for(int x = 0; x < base.cols*ch; x++)
		{
			int ix = x / ch;
			
				 if(pMask[ix] == 255) pOut[x] = pBase[x];
			else if(pMask[ix] == 0)   pOut[x] = 0;
			else
			{
				alpha = pMask[ix] / 255.0f;
				pOut[x] = uchar(pBase[x] * alpha);
			}
		}
	}
}

