/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../stdafx.h"

#include "Img2Cpp.h"

using namespace std;
using namespace cv;

void Img2Cpp::createCppFromImage_PNG(
	string image_file, string name,
	string out_file, bool append_file)
{
	// Load image as color image
	Mat image = imread(image_file, CV_LOAD_IMAGE_COLOR);
	if(image.empty()) throw runtime_error(
		"Img2Cpp: Failed to load image file '" + image_file + "'.");

	// Setup PNG compression parameters
	// to apply highst compression rate
	vector<int> params(2);
	params[0] = CV_IMWRITE_PNG_COMPRESSION;
	params[1] = 9;

	// Encode the image as PNG and store in a vector as buffer
	std::vector<unsigned char> encoded_image_buffer;
	imencode(".png", image, encoded_image_buffer, params);
	
	// Get the size of the vector
	size_t data_size = encoded_image_buffer.size();

	// Open output file for writing C++ code to.
	// If indicated, append file instead of overwriting.
	ios_base::openmode mode = ios_base::out;
	if(append_file) mode |= ios_base::app;
	ofstream code_file(out_file, mode); if(!code_file.good())
		throw runtime_error("Img2Cpp: Unable to open file '" + out_file +"' for writing.") ;

	// ----------------------------------------
	// Create C++ code that contains image data
	// ----------------------------------------

	// Declare a namespace
	code_file << "namespace Img2Cpp {" << endl;
	
	// Declare a variable storing the data length
	code_file << "unsigned int " << name << "_length = ";
	code_file << (unsigned int)data_size << ";" << endl;

	// Declare an array of unsigned chars
	code_file << "unsigned char " << name << "[] = {";

	// Write all data as initialization of unsigned
	// char array. Add a newline every N values.
	for(size_t i = 0; i < data_size; i++)
	{
		if(i % 30 == 0) code_file << endl;
		code_file << format("0x%02x", encoded_image_buffer[i]);
		if(i != (data_size - 1)) code_file << ",";
	}

	// Close array delaration
	code_file << endl << "};" << endl;

	// Close namespace declaration
	code_file << "}" << endl << endl;
}

Mat Img2Cpp::loadImageFromArray_PNG(
	unsigned char data[], size_t data_size)
{
	// Assign the data to a const vector as buffer
	const std::vector<unsigned char> encoded_image_buffer(
		data, data + data_size);

	// Decode the image from the buffer
	Mat image = imdecode(encoded_image_buffer, CV_LOAD_IMAGE_COLOR);
	if(image.empty()) throw runtime_error(
		"Img2Cpp: Failed to load image from C++ code.");

	return image;
}
