/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../stdafx.h"

#include "ThreadCout.h"

using namespace std;

ThreadCout ThreadCout::tcout;

ThreadCout::ThreadCout()
	: one_second(1000000000LL)
{
	m_timer.start();
}

void ThreadCout::setThreadName(string name)
{
	boost::thread::id threadID = boost::this_thread::get_id();
	m_threadNameMap[threadID] = name;
}

string ThreadCout::getCurrThreadName()
{
	boost::thread::id threadID = boost::this_thread::get_id();
	return getThreadName(threadID);
}

string ThreadCout::getThreadName(boost::thread::id threadID)
{
	string &name = m_threadNameMap[threadID];
			
	if(name.empty())
	{
		stringstream builder;
		builder << "Thd " << threadID;
		name = builder.str();
	}

	return name;
}

void ThreadCout::display(const std::ostream &msg)
{
	stringstream sstr;
	sstr << msg.rdbuf();

	display(sstr.str());
}

void ThreadCout::display(const string msg)
{
	// Loads a message into the queue
	
	Msg msgObj;

	msgObj.threadID = boost::this_thread::get_id();
	msgObj.time = m_timer.elapsed().wall;
	msgObj.msg = msg;

	m_queue.Enqueue(msgObj);
}

void ThreadCout::render(ostream &os, bool includeThreadNames)
{
	// Prints the message to the output stream
	vector<Msg> messages;
	while(m_queue.getSize() > 0)
		messages.push_back(m_queue.Dequeue());

	sort(messages.begin(), messages.end());

	for(Msg &msg : messages)
	{
		os << "[" << msg.time/one_second << "s] ";

		if( includeThreadNames )
		{
			string name = getThreadName(msg.threadID);

			os << "[" << name << "] ";
		}

		os << msg.msg << endl;
	}
}



#ifndef WIN32

// extensions start
/*! \brief define a simple ansi console colors struct
 *
 * The contained values correspond to ansi color definitions
 * However, the colors are only work on platforms that are
 * able to interpret these ansi escape sequences, like
 * [L|U]nix
 */
struct ConsoleColors {
	enum Colors {
		reset = 0,
		black=30,
		red=31,
		green=32,
		yellow=33,
		blue = 34,
		magenta=35,
		cyan=36,
		white=37
	};
};

#define GENERATE_OPERATOR_CONTENT   \
		unsigned char tmp=Base::getBase();  \
		*this << ::logging::log::dec << "\033[" << static_cast<unsigned short>(l) << 'm';  \
		Base::setBase(tmp);

#else

#include <windows.h> // WinApi header

/*! \brief define a simple console colors struct for windows terminals
 */
struct ConsoleColors {
	enum Colors {
		reset = 7,
		blue=9,
		green,
		cyan,
		red,
		magenta,
		yellow,
		white,
		black=0
	};
};

#define GENERATE_OPERATOR_CONTENT   \
	HANDLE hConsole; \
	hConsole = GetStdHandle(STD_OUTPUT_HANDLE); \
	SetConsoleTextAttribute(hConsole, l);

#endif