/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#ifndef __THREAD_COUT__
#define __THREAD_COUT__

#include "SynchronizedQueue.h"

class ThreadCout
{
private:
	struct Msg
	{
		boost::thread::id threadID;
		boost::timer::nanosecond_type time;
		std::string msg;

		inline bool operator< (const Msg &rhs) const
		{	return time < rhs.time;		}
	};

	typedef std::map<boost::thread::id, std::string> NameMap;

	SynchronizedQueue<Msg> m_queue ; // Use SynchronizedQueue to store messages
	boost::timer::cpu_timer m_timer;

	NameMap m_threadNameMap;

	boost::timer::nanosecond_type const one_second ; 

public:
	ThreadCout();

	void setThreadName(std::string name);
	std::string getThreadName(boost::thread::id threadID);
	
	std::string getCurrThreadName();

	void display(const std::ostream &msg);
	void display(const std::string msg);

	unsigned int getPendingMsgCount()
	{	return m_queue.getSize();	}

	void render(std::ostream &os = std::cout,
				bool includeThreadNames = false);

	static ThreadCout tcout;
} ;

// Macro for ease of use
#define TLOG(m) { ThreadCout::tcout.display(std::stringstream() << m); }

#endif //__THREAD_COUT__
