/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../stdafx.h"

#include "Util.h"

// For date and time
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/date_time/posix_time/posix_time_io.hpp>

using namespace std;

string::size_type
Util::findLastDirectorySeperator(const string file)
{
	string::size_type lastSlash = file.find_last_of("/");
	string::size_type lastBackslash = file.find_last_of("\\");

	if(lastSlash == string::npos) return lastBackslash;
	else if(lastBackslash == string::npos) return lastSlash;
	else return min(lastSlash, lastBackslash);
}

void
Util::splitFilePathAndName(const string file,
	string &path, string &name)
{
	string::size_type lastSeperator = 
		findLastDirectorySeperator(file);

	if(lastSeperator == string::npos)
	{
		path = "";
		name = file;
	}
	else
	{
		path = file.substr(0, lastSeperator + 1);
		name = file.substr(lastSeperator + 1, -1);
	}
}

void
Util::writeDateTime(std::ostream &os)
{
	boost::posix_time::time_facet facet("%d-%b-%Y %H:%M:%S");
		
	stringstream timeOut ;
	timeOut.imbue(locale(cout.getloc(), &facet)) ;
		
	timeOut << boost::posix_time::second_clock::local_time() << endl ;
		
	os << timeOut.str() ;
}

void
Util::stdOut_goUpByOneLine()
{
// Windows version
#ifdef _WIN32
	HANDLE hStdConsole = GetStdHandle( STD_OUTPUT_HANDLE );

	CONSOLE_SCREEN_BUFFER_INFO info;
	GetConsoleScreenBufferInfo(hStdConsole, &info);

	COORD currPos = info.dwCursorPosition;
	if(currPos.Y > 0) currPos.Y--;

	SetConsoleCursorPosition(hStdConsole, currPos);
	
// ANSI escape code version
#else
	cout << "\x1b[A";

#endif
}
