/*
	Copyright (C) 2015  University of Mannheim

	This file is part of CamMark.

	CamMark is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CamMark is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CamMark.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#ifndef __VIDEOREADER__
#define __VIDEOREADER__

class VideoReader
{
public:
	// Construction
	VideoReader();
	VideoReader(int device);
	VideoReader(const std::string &filename);
	virtual ~VideoReader(void);

	// Opening/closing
	virtual bool open(int device);
	virtual bool open(const std::string &filename);
	virtual void close();
	
	virtual bool isOpened();

	// General properties get/set methods
	virtual double get(int propId);
	virtual bool set(int propId, double value);

	// Special FPS get/set method
	virtual double getFPS();
	virtual double setFPS(double value);

	// Get media size
	virtual cv::Size getSize()
	{	return size;	}
	
	// Get frame count
	virtual unsigned int getFrameCount()
	{	return frameCount;	}

	// Seeking
	virtual unsigned int seekToFrame(unsigned int frame_nr);
	
	// Frame reading
	inline bool read(cv::Mat& image)
	{	return cv_capture.read(image);	}

	// Access through >> operator
	inline VideoReader& operator>>(cv::Mat& image)
	{
		cv_capture.read(image);
		return *this;
	}

protected:
	// Utility function
	static void replaceSubstring(std::string& str,
		const std::string& oldStr,
		const std::string& newStr);

protected:
	// Internal OpenCV VideoCapture object
	cv::VideoCapture cv_capture;

	// Internal fallback FPS field
	double fps;

	// Internal size field
	cv::Size size;

	// Internal frame count field
	unsigned int frameCount;

private:
	// The max number of digits in a filename
	// pattern when reading still image sequences
	static unsigned int MAX_NUM_DIGITS;
};

#endif // __VIDEOREADER__